from time import clock

# Matrix 2x2 is a represented as tuple of tuples
# | a b |
# | c d |  = ((a,b), (c,d))

def m2x2_mulp(A, B):
	return ((
		A[0][0]*B[0][0] + A[0][1]*B[1][0],
		A[0][0]*B[0][1] + A[0][1]*B[1][1]
	), (
		A[1][0]*B[0][0] + A[1][1]*B[1][0],
		A[1][0]*B[0][1] + A[1][1]*B[1][1]
	))


def m2x2_pow(A, n):
	if n == 0:
		return ((1,0), (0,1))  # identity
	if n == 1:
		return A  # base
	if (n%2 == 0):
		P = m2x2_pow(A, n/2)
		return m2x2_mulp(P, P)
	else:
		P = m2x2_pow(A, (n-1)/2)
		return m2x2_mulp(m2x2_mulp(P, P), A)
# T(n) = T(n/2) + c = Theta(log n)


I = ((1,1), (1,0))
def fib3(n):
	M = m2x2_pow(I, n)
	return M[0][1]


for n in range(1000, 1000000, 1000):
	start = clock()
	r = fib3(n)
	end = clock()
	print('fib3({})={}\t\tin\t\t{}sec'.format(n, -1, end-start))
